#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <string.h>
#include "misc.h"
#include <rtdk.h>

/************************************************************************/
/* a = pointer to array, n = number of elements				*/
/*									*/
/* Function to sort an array     					*/
/************************************************************************/
void bubbleSort(float *a, unsigned int n)
{
  unsigned int i = 0, j = 0;
  float tmp;
  for (i=0; i<n-1; i++) {
    for (j=0; j<n-1-i; j++)
      if (a[j+1] < a[j]) {  /* compare the two neighbors */
	tmp = a[j];         /* swap a[j] and a[j+1]      */
	a[j] = a[j+1];
	a[j+1] = tmp;
      }
  }
}


//************************************************************************/

float centroid(float *ps, int r, int c, int cpeak, float wl, float bw, float *  wt)
{
  int i,j;
  float minps=1e99;
  float xc = 0.;
  float yc = 0.;

  float *sortit;
  sortit = (float *)malloc(r*sizeof(float));  

  for (j = cpeak - 1;j <  cpeak + 2; j++){
    for (i=0; i<r; i++){
      minps=MIN(minps,*(ps+j+i*c));
      sortit[i]=*(ps+j+i*c);
    }
    // Use Median instead.
      bubbleSort(sortit, r);
      minps = sortit[ (r / 2) -1 ];

    for (i = 0; i<r/2; i++){
      xc += (*(ps + j + i*c) - minps) *cos(i*2*PI/r)/minps;
      yc += (*(ps + j + i*c) - minps) *sin(i*2*PI/r)/minps;
      
      xc += ( *(ps + j + i*c + r/2*c) - minps) *cos(i*2*PI/r - PI)/minps;
      yc += ( *(ps + j + i*c + r/2*c) - minps) *sin(i*2*PI/r - PI)/minps;
    }
  }
  *wt=xc*xc+yc*yc / 3. ;  // Norm by number of sum-cols (3)
  free(sortit);
  return( (0.5*(float)r/PI) *atan2f(yc, xc) * wl*wl /bw );
}


float centroid_range(float *ps, int r, int c, int cpeak0, int cpeak1, 
              float wl, float bw, float *  wt)
{
  int i,j;
  float minps=1e99;
  float xc = 0.;
  float yc = 0.;

  float *sortit;
  sortit = (float *)malloc(r*sizeof(float));  

  for (j = cpeak0;j < cpeak1+1; j++){
    for (i=0; i<r; i++){
      minps=MIN(minps,*(ps+j+i*c));
      sortit[i]=*(ps+j+i*c);
    }
    // Use Median instead.
      bubbleSort(sortit, r);
      minps = sortit[ (r / 2) -1 ];

    for (i = 0; i<r/2; i++){
      xc += (*(ps + j + i*c) - minps) *cos(i*2*PI/r)/minps;
      yc += (*(ps + j + i*c) - minps) *sin(i*2*PI/r)/minps;
      
      xc += ( *(ps + j + i*c + r/2*c) - minps) *cos(i*2*PI/r - PI)/minps;
      yc += ( *(ps + j + i*c + r/2*c) - minps) *sin(i*2*PI/r - PI)/minps;
    }
  }
  *wt=xc*xc+yc*yc/ ( (float)(cpeak1-cpeak0+1));  // Norm by number of sum-cols
  free(sortit);
  return( (0.5*(float)r/PI) *atan2f(yc, xc) * wl*wl /bw );
}





/* only for 4 telescopes */
void bootStrapping(float w[], float OPD0[], float wMeanOPD[])
{
  float a, b, c, sum1, sum2;
  float OPD[6];
  int i;

  // w = [w21(0), w31(1), w41(2), w32(3), w42(4), w43(5)]
  // OPD = [OPD21, OPD31, OPD41, OPD32, OPD42, OPD43]
  // wMeanOPD = [wMeanOPD21, wMeanOPD31, wMeanOPD41]

  // zero out the OPDS for weak fringes.
  for (i=0;i<6;i++) {
    if (w[i] <= 3.0) OPD[i]=0.0; else OPD[i]=OPD0[i];
  }

  // OPD21  
  a = w[0];
  b = (w[1]+w[3]) <= 0. ? 0. : w[1]*w[3]/(w[1]+w[3]);
  c = (w[2]+w[4]) <= 0. ? 0. : w[2]*w[4]/(w[2]+w[4]);
  sum1 = a + b + c;
  sum2 = a*OPD[0] + b*(OPD[1] - OPD[3]) + c*(OPD[2] - OPD[4]);
  wMeanOPD[0] = (sum1 <= 0.) ? 0. :sum2/sum1;

  // OPD31
  a = w[1];
  b = (w[0]+w[3]) <= 0. ? 0. : w[0]*w[3]/(w[0]+w[3]);
  c = (w[5]+w[2]) <= 0. ? 0. : w[5]*w[2]/(w[5]+w[2]);
  sum1 = a + b + c;
  sum2 = a*OPD[1] + b*(OPD[0]+OPD[3]) + c*(OPD[2]-OPD[5]);
  wMeanOPD[1] = (sum1 <= 0.) ? 0. :sum2/sum1;

  // OPD41
  a = w[2];
  b = (w[0]+w[4]) <= 0. ? 0. : w[0]*w[4]/(w[0]+w[4]);
  c = (w[1]+w[5]) <= 0. ? 0. : w[1]*w[5]/(w[1]+w[5]);
  sum1 = a + b + c;
  sum2 = a*OPD[2] + b*(OPD[0]+OPD[4]) + c*(OPD[1]+OPD[5]);
  wMeanOPD[2] = (sum1 <= 0.) ? 0. :sum2/sum1;

}

/* only for 4 telescopes --  JDM 2005Sep11*/
void bootStrapping4(float w[], float OPD0[], float wMeanOPD[])
{
  float a11,a12,a13,a21,a22,a23,a31,a32,a33;
  float w12,w13,w14,w23,w24,w34;
  float opd12,opd13,opd14,opd23,opd24,opd34;
  float i11,i12,i13,i21,i22,i23,i31,i32,i33,deta;
  float b1,b2,b3;
  float OPD[6];
  int i;

  // w = [w21(0), w31(1), w41(2), w32(3), w42(4), w43(5)]
  // OPD = [OPD21, OPD31, OPD41, OPD32, OPD42, OPD43]
  // wMeanOPD = [wMeanOPD21, wMeanOPD31, wMeanOPD41]

   // zero out the OPDS for weak fringes.
  for (i=0;i<6;i++) {
    if (w[i] <= 1.0) {
      OPD[i]=0.0;
    } else OPD[i]=OPD0[i];
  }
  opd12=OPD[0];
  opd13=OPD[1];
  opd14=OPD[2];
  opd23=OPD[3];
  opd24=OPD[4];
  opd34=OPD[5];

  w12=MAX(w[0],.01);
  w13=MAX(w[1],.01);
  w14=MAX(w[2],.01);
  w23=MAX(w[3],.01);
  w24=MAX(w[4],.01);
  w34=MAX(w[5],.01);
  
  b1=opd12*w12-opd23*w23-opd24*w24;
  b2=opd13*w13+opd23*w23-opd34*w34;
  b3=opd14*w14+opd24*w24+opd34*w34;

  a11=w12+w23+w24;
  a12=-w23;
  a13=-w24;
  a21=a12;
  a22=w13+w23+w34;
  a23=-w34;
  a31=a13;
  a32=a23;
  a33=w14+w24+w34;

  i11=a33*a22-a32*a23;
  i12=-1*(a33*a12-a32*a13);
  i13=a23*a12-a22*a13;
  i21=-1*(a33*a21-a31*a23);
  i22=a33*a11-a31*a13;
  i23=-1*(a23*a11-a21*a13);
  i31=a32*a21-a31*a22;
  i32=-1*(a32*a11-a31*a12);
  i33=a22*a11-a21*a12;

  deta=a11*(a33*a22-a32*a23)-a21*(a33*a12-a32*a13)+a31*(a23*a12-a22*a13);

  wMeanOPD[0]=(i11*b1 +i12*b2 + i13*b3)/deta;
  wMeanOPD[1]=(i21*b1 +i22*b2 + i23*b3)/deta;
  wMeanOPD[2]=(i31*b1 +i32*b2 + i33*b3)/deta;

}

int *ivector(long nl, long nh)
{
int *v;
v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
if (!v) rt_printf("allocation failure in ivector()");
return v-nl+NR_END;
}

void free_ivector(int *v, long nl, long nh)
{
free((FREE_ARG) (v+nl-NR_END));
}


void bootStrapping6(float w[], float OPD0[], float wMeanOPD[])
{
  float a[5][5],b[5][1];
  float w12,w13,w14,w15,w16,w23,w24,w25,w26,w34,w35,w36,w45,w46,w56;
  float opd12,opd13,opd14,opd15,opd16,opd23,opd24,opd25,opd26,opd34,opd35,opd36,opd45,opd46,opd56;
  float OPD[15];
  int n=5,m=1;                   //matrix array dimension
  int i,icol,irow,j,k,l,ll;
  int *indxc,*indxr,*ipiv;
  float big,dum,pivinv,temp;

  for (i=0;i<15;i++) {
    if (w[i] <= 1.0) {
      OPD[i]=0.0;
    } else OPD[i]=OPD0[i];
  }
  opd12=OPD[0];
  opd13=OPD[1];
  opd14=OPD[2];
  opd15=OPD[3];
  opd16=OPD[4];
  opd23=OPD[5];
  opd24=OPD[6];
  opd25=OPD[7];
  opd26=OPD[8];
  opd34=OPD[9];
  opd35=OPD[10];
  opd36=OPD[11];
  opd45=OPD[12];
  opd46=OPD[13];
  opd56=OPD[14];

  /*opd12=-6.;
  opd13=-9.;
  opd14=1.;
  opd15=-8.;
  opd16=-1.;
  opd23=-3.;
  opd24=7.;
  opd25=-2.;
  opd26=5.;
  opd34=10.;
  opd35=1.;
  opd36=8.;
  opd45=-9.;
  opd46=-2.;
  opd56=7.;*/

  w12=MAX(w[0],.01)+0.001;
  w13=MAX(w[1],.01)+0.002;
  w14=MAX(w[2],.01)+0.005;
  w15=MAX(w[3],.01)+0.007;
  w16=MAX(w[4],.01)+0.003;
  w23=MAX(w[5],.01)+0.004;
  w24=MAX(w[6],.01)+0.008;
  w25=MAX(w[7],.01)+0.009;
  w26=MAX(w[8],.01)+0.002;
  w34=MAX(w[9],.01)+0.003;
  w35=MAX(w[10],.01)+0.006;
  w36=MAX(w[11],.01)+0.008;
  w45=MAX(w[12],.01)+0.009;
  w46=MAX(w[13],.01)+0.004;
  w56=MAX(w[14],.01)+0.005;

  /*w12=MAX(w[0],.01)+10000;
  w13=MAX(w[1],.01)+13000;
  w14=MAX(w[2],.01)+45000;
  w15=MAX(w[3],.01)+56000;
  w16=MAX(w[4],.01)+32000;
  w23=MAX(w[5],.01)+17000;
  w24=MAX(w[6],.01)+98000;
  w25=MAX(w[7],.01)+26000;
  w26=MAX(w[8],.01)+77000;
  w34=MAX(w[9],.01)+59000;
  w35=MAX(w[10],.01)+12000;
  w36=MAX(w[11],.01)+52000;
  w45=MAX(w[12],.01)+84000;
  w46=MAX(w[13],.01)+73000;
  w56=MAX(w[14],.01)+59000;*/


  a[0][0] = w12+w23+w24+w25+w26;
  a[1][1] = w13+w23+w34+w35+w36;
  a[2][2] = w14+w24+w34+w45+w46;
  a[3][3] = w15+w25+w35+w45+w56;
  a[4][4] = w16+w26+w36+w46+w56;

  a[0][1] = -w23;
  a[0][2] = -w24;
  a[0][3] = -w25;
  a[0][4] = -w26;

  a[1][0] = -w23;
  a[1][2] = -w34;
  a[1][3] = -w35;
  a[1][4] = -w36;

  a[2][0] = -w24;
  a[2][1] = -w34;
  a[2][3] = -w45;
  a[2][4] = -w46;

  a[3][0] = -w25;
  a[3][1] = -w35;
  a[3][2] = -w45;
  a[3][4] = -w56;

  a[4][0] = -w26;
  a[4][1] = -w36;
  a[4][2] = -w46;
  a[4][3] = -w56;

  b[0][0] = w12*opd12 - w23*opd23 - w24*opd24 - w25*opd25 - w26*opd26;
  b[1][0] = w13*opd13 + w23*opd23 - w34*opd34 - w35*opd35 - w36*opd36;
  b[2][0] = w14*opd14 + w24*opd24 + w34*opd34 - w45*opd45 - w46*opd46;
  b[3][0] = w15*opd15 + w25*opd25 + w35*opd35 + w45*opd45 - w56*opd56;
  b[4][0] = w16*opd16 + w26*opd26 + w36*opd36 + w46*opd46 + w56*opd56;

//  for(i=0;i<5;i++)
//    {
//      for(j=0;j<5;j++)
//  rt_printf("%f ",a[i][j]);
//     rt_printf("\n");
//   }
//  for(i=0;i<5;i++) rt_printf("%f ",b[i][0]);

//  rt_printf("\n");
  //start solving the linear equation
  // adopted from numerical receipt in c, gaussj.c ---XC

indxc=ivector(1,n);
indxr=ivector(1,n);
ipiv=ivector(1,n);
for (j=0;j<n;j++) ipiv[j]=0;
for (i=0;i<n;i++) {
        big=0.0;
        for (j=0;j<n;j++)
                if (ipiv[j] != 1)
                        for (k=0;k<n;k++) {
                                if (ipiv[k] == 0) {
                                        if (fabs(a[j][k]) >= big) {
                                                big=fabs(a[j][k]);
                                                irow=j;
                                                icol=k;
                                        }
                                }
                        }
        ++(ipiv[icol]);
        if (irow != icol) {
                for (l=0;l<n;l++) SWAP(a[irow][l],a[icol][l])
                for (l=0;l<m;l++) SWAP(b[irow][l],b[icol][l])
        }
        indxr[i]=irow;
        indxc[i]=icol;
        if (a[icol][icol] != 0.0) //printf("Singular Matrix");
	  pivinv=1.0/a[icol][icol];
	else
	  pivinv =0;
        a[icol][icol]=1.0;
        for (l=0;l<n;l++) a[icol][l] *= pivinv;
        for (l=0;l<m;l++) b[icol][l] *= pivinv;
        for (ll=0;ll<n;ll++)
                if (ll != icol) {
                        dum=a[ll][icol];
                        a[ll][icol]=0.0;
                        for (l=0;l<n;l++) a[ll][l] -= a[icol][l]*dum;
                        for (l=0;l<m;l++) b[ll][l] -= b[icol][l]*dum;
                }
}
for (l=n-1;l>=0;l--) {
        if (indxr[l] != indxc[l])
                for (k=0;k<n;k++)
                        SWAP(a[k][indxr[l]],a[k][indxc[l]]);
}
free_ivector(ipiv,1,n);
free_ivector(indxr,1,n);
free_ivector(indxc,1,n);

 wMeanOPD[0]=b[0][0];
 wMeanOPD[1]=b[1][0];
 wMeanOPD[2]=b[2][0];
 wMeanOPD[3]=b[3][0];
 wMeanOPD[4]=b[4][0];

}

