/*
 * spooler.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define NAXIS_LEN       3
#define GPS             "gps"
#define OPLE            "ople"
#define TIPTILT         "tiptilt"
#define SPOOLER         "SPOOLER"
#define STATUSMSG       "status"
#define PORT            2200
#define ALTPORT         4000
#define BUFLEN          2000
#define LLENGTH         19
#define NUM_THREADS	10
#define SLEEPTIME       1000000
#define NSTACK 1000
#define MEGEXTRA 1000//1000000
#define SOCKET_BUFFER 65536 // from /libs/charaui/src/charaui.h (needed for vme socket)

/* Format of status message from tiptilt server */

/* Local Tm:  16:41:33         SUM   DETrms MIRrms SERVO MOD SIN r0(cm) Seeing Snd */
/* CHARA Tm:  22:30:37    B1 0126130  0.9437 0.000  OFF       NO    0.0   0.00  ON */
/* Lost T/S:      0/XX    B2 0060242  0.258  0.000  OFF       NO    0.0   0.00  ON */
/* ExpTm mS:         1    B3 0075723  0.669  0.000  OFF       NO    0.0   0.00  ON */
/* Frms/Sec:    464.00    B4 0066127  0.847  0.000  OFF       NO    0.0   0.00  ON */
/* Frame Tp: SUB_FRAME    B5 0037738  0.2901 0.000  OFF       NO    0.0   0.00 OFF */
/* Frame Ps:    0    0    B6 0035685  0.6595 0.000  OFF       NO    0.0   0.00 OFF */
/* Frame Sz:   20   18    SIGNALS:                                              ON */
/* Npixels :       360 */
/* COMMAND_COMPLETE:0 */


/** Delay lines vector indexes*/
enum {
  S1_TARGET,
  S2_TARGET,
  E1_TARGET,
  E2_TARGET,
  W1_TARGET,
  W2_TARGET,
};


/* Temporary definitions for testing */
#define TIMERINT 30
#define MAX_SELECT_FD 20
#define SELECT_TIMEOUT 2*1000


static const char* mon[12] = {
  "Jan",
  "Feb",
  "Mar",
  "Apr",
  "May",
  "Jun",
  "Jul",
  "Aug",
  "Sep",
  "Oct",
  "Nov",
  "Dec"
};

/* Sockets' file descriptors */
enum SOCK {
  STARFD,
  UVFD,
  GPSFD,
  TIPTILTFD,
/*   SHUTTERFD, */
  SPOOLFD
};

typedef enum SOCK tcpsock;

typedef void *(*PTF) (void *);

/* Structure to initialise threads */
struct thread_data {
  tcpsock  socknum;
  char hostname[30];
  int port;
  int fd;
  PTF func;
  void *message;
  void *messageBack;
  void *ptr;
};
typedef struct thread_data threadData;


/*****************************************************/
/* function prototipes                               */
/*****************************************************/
void cleanExit(int dummy);
void printerror( int status);
void openServer(void *host);
int queryServer(int sockfd, void *serverarg);
int readServer(int sockfd, void *serverarg);
char *fmtString(char *buffer);
void *socketThread(void *threadarg);
void *spoolThread(void *threadarg);
